from setuptools import setup, find_packages

setup(
    name = "openproximity-external-media",
    version = "0.1.3",
    packages = ['openproximity_ext'],
    include_package_data = True,
    author = "Naranjo Manuel Francisco",
    author_email = "manuel@aircable.net",
    description = "This is OpenProximity external metadata package",
    keywords = "openproximity proximity marketing bluetooth static",
    url = "https://github.com/OpenProximity/OpenProximity-web-3rdparty-static",
)
